#ifndef _NET_TUX_TUX_U_H
#define _NET_TUX_TUX_U_H

/*
 * TUX - Scalable Integrated Web Cache and Web Server
 *
 * Copyright (C) 2000 Red Hat, Inc.
 * Written by Ingo Molnar <mingo@redhat.com>
 *
 * tux.h: TUXAPI - TUX interface to user-space
 */

#define __KERNEL_SYSCALLS__

typedef enum http_versions {
        HTTP_1_0,
        HTTP_1_1
} http_version_t;

/*
 * Request methods known to TUX:
 */
typedef enum http_methods {
        METHOD_NONE,
        METHOD_GET,
        METHOD_HEAD,
        METHOD_POST,
        METHOD_PUT
} http_method_t;

enum user_req {
	TUX_ACTION_STARTUP = 1,
	TUX_ACTION_SHUTDOWN = 2,
	TUX_ACTION_STARTTHREAD = 3,
	TUX_ACTION_STOPTHREAD = 4,
	TUX_ACTION_EVENTLOOP = 5,
	TUX_ACTION_GET_OBJECT = 6,
	TUX_ACTION_SEND_OBJECT = 7,
	TUX_ACTION_READ_OBJECT = 8,
	TUX_ACTION_FINISH_REQ = 9,
	TUX_ACTION_REGISTER_MODULE = 10,
	TUX_ACTION_UNREGISTER_MODULE = 11,
	MAX_TUX_ACTION
};

enum tux_ret {
	TUX_RETURN_USERSPACE_REQUEST = 0,
	TUX_RETURN_EXIT = 1,
	TUX_RETURN_SIGNAL = 2,
};

#define MAX_MODULENAME_LEN 16
#define MAX_URI_LEN 256
#define MAX_POST_DATA 1024
#define MAX_COOKIE_LEN 128

typedef struct user_req_s {
	int http_version;
	int http_method;
	int sock;
	int bytes_sent;
	int http_status;
	unsigned int client_host;
	unsigned int objectlen;
	char query[MAX_URI_LEN];
	char *object_addr;
	char objectname[MAX_URI_LEN];
	int module_index;
	char modulename[MAX_MODULENAME_LEN];
	char post_data[MAX_POST_DATA];

	int cookies_len;
	char cookies[MAX_COOKIE_LEN];

	int event;
	int thread_nr;
	void *id;
	void *private;
} user_req_t;

extern char *TUXAPI_docroot;
extern char *TUXAPI_version;

extern int tux (unsigned int action, user_req_t *req);
extern void * TUXAPI_malloc_shared (unsigned int len);

#define BUG()								\
do {									\
	printf("CAD BUG at %d:%s!\n", __LINE__, __FILE__);		\
	tux(TUX_ACTION_STOPTHREAD, NULL);				\
	tux(TUX_ACTION_SHUTDOWN, NULL);				\
	*(int*)0=0;							\
	exit(-1);							\
} while (0)

#define LOCK_PREFIX "lock ; "

#define barrier() __asm__ __volatile__("": : :"memory")
struct __dummy { unsigned long a[100]; };
#define ADDR (*(volatile struct __dummy *) addr)

extern __inline__ int test_and_set_bit(int nr, volatile void * addr)
{
	int oldbit;

	__asm__ __volatile__( LOCK_PREFIX
		"btsl %2,%1\n\tsbbl %0,%0"
		:"=r" (oldbit),"=m" (ADDR)
		:"Ir" (nr));
	return oldbit;
}

extern __inline__ void tux_down (int *sem)
{
	while (test_and_set_bit(0, sem))
		barrier();
}

extern __inline__ void tux_up (int *sem)
{
	*((volatile int *)sem) = 0;
}

#endif
